@@ -1,39 +1,38 @@
 /************************************************************************
+ 
+ IMPORTANT NOTE : this file contains two clearly delimited sections :
+ the ARCHITECTURE section (in two parts) and the USER section. Each section
+ is governed by its own copyright and license. Please check individually
+ each section for license and copyright information.
+ *************************************************************************/
 
-    IMPORTANT NOTE : this file contains two clearly delimited sections :
-    the ARCHITECTURE section (in two parts) and the USER section. Each section
-    is governed by its own copyright and license. Please check individually
-    each section for license and copyright information.
-*************************************************************************/
-
-/*******************BEGIN ARCHITECTURE SECTION (part 1/2)****************/
-
+/******************* BEGIN max-msp64.cpp ****************/
 /************************************************************************
-    FAUST Architecture File
-    Copyright (C) 2004-2020 GRAME, Centre National de Creation Musicale
-    ---------------------------------------------------------------------
-    This Architecture section is free software; you can redistribute it
-    and/or modify it under the terms of the GNU Lesser General Public
-    License as published by the Free Software Foundation; either version 3
-    of the License, or (at your option) any later version.
-
-    This program is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-    GNU Lesser General Public License for more details.
-
-    You should have received a copy of the GNU Lesser General Public License
-    along with this program; If not, see <http://www.gnu.org/licenses/>.
-
-    EXCEPTION : As a special exception, you may create a larger work
-    that contains this FAUST architecture section and distribute
-    that work under terms of your choice, so long as this FAUST
-    architecture section is not modified.
-
-    MAX MSP SDK : in order to compile a MaxMSP external with this
-    architecture file you will need the official MaxMSP SDK from
-    cycling'74. Please check the corresponding license.
-
+ FAUST Architecture File
+ Copyright (C) 2004-2020 GRAME, Centre National de Creation Musicale
+ ---------------------------------------------------------------------
+ This Architecture section is free software; you can redistribute it
+ and/or modify it under the terms of the GNU Lesser General Public
+ License as published by the Free Software Foundation; either version 3
+ of the License, or (at your option) any later version.
+ 
+ This program is distributed in the hope that it will be useful,
+ but WITHOUT ANY WARRANTY; without even the implied warranty of
+ MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ GNU Lesser General Public License for more details.
+ 
+ You should have received a copy of the GNU Lesser General Public License
+ along with this program; If not, see <http://www.gnu.org/licenses/>.
+ 
+ EXCEPTION : As a special exception, you may create a larger work
+ that contains this FAUST architecture section and distribute
+ that work under terms of your choice, so long as this FAUST
+ architecture section is not modified.
+ 
+ MAX MSP SDK : in order to compile a MaxMSP external with this
+ architecture file you will need the official MaxMSP SDK from
+ cycling'74. Please check the corresponding license.
+ 
  ************************************************************************
  ************************************************************************/
 
@@ -67,6 +66,16 @@
 
 // FAUSTFLOAT is setup by faust2max6
 
+#ifndef DOWN_SAMPLING
+#define DOWN_SAMPLING 0
+#endif
+#ifndef UP_SAMPLING
+#define UP_SAMPLING 0
+#endif
+#ifndef FILTER_TYPE
+#define FILTER_TYPE 0
+#endif
+
 #include "faust/gui/UI.h"
 #include "faust/gui/SimpleParser.h"
 #include "faust/gui/PathBuilder.h"
@@ -133,7 +142,7 @@ using namespace std;
 #define ASSIST_INLET 	1
 #define ASSIST_OUTLET 	2
 
-#define EXTERNAL_VERSION    "0.77"
+#define EXTERNAL_VERSION    "0.87"
 #define STR_SIZE            512
 
 #include "faust/gui/GUI.h"
@@ -141,7 +150,7 @@ using namespace std;
 #include "faust/gui/mspUI.h"
 #include "faust/dsp/poly-dsp.h"
 
-std::list<GUI*> GUI::fGuiList;
+list<GUI*> GUI::fGuiList;
 ztimedmap GUI::gTimedZoneMap;
 
 static t_class* faust_class;
@@ -164,7 +173,6 @@ typedef struct faust
     void** m_args;
     mspUI* m_dspUI;
     dsp* m_dsp;
-    mydsp_poly* m_dsp_poly;
     void* m_control_outlet;
     char* m_json;
     t_systhread_mutex m_mutex;    
@@ -173,7 +181,8 @@ typedef struct faust
     SaveUI* m_savedUI;
 #ifdef MIDICTRL
     MidiUI* m_midiUI;
-    midi_handler* m_midiHandler;
+    max_midi* m_midiHandler;
+    void* m_midi_outlet;
 #endif
 #ifdef SOUNDFILE
     SoundUI* m_soundInterface;
@@ -191,56 +200,23 @@ void faust_allocate(t_faust* x, int nvoices)
 {
     // Delete old
     delete x->m_dsp;
-    x->m_dspUI->clear();
+    delete x->m_dspUI;
+    x->m_dspUI = new mspUI();
     
     if (nvoices > 0) {
     #ifdef POST
         post("polyphonic DSP voices = %d", nvoices);
     #endif
-        x->m_dsp_poly = new mydsp_poly(new mydsp(), nvoices, true, true);
+        x->m_dsp = new mydsp_poly(new mydsp(), nvoices, true, true);
     #ifdef POLY2
-        x->m_dsp = new dsp_sequencer(x->m_dsp_poly, new effect());
-    #else
-        x->m_dsp = x->m_dsp_poly;
-    #endif
-    #ifdef MIDICTRL
-        x->m_midiHandler->addMidiIn(x->m_dsp_poly);
+        x->m_dsp = new dsp_sequencer(x->m_dsp, new effect());
     #endif
     } else {
     #ifdef POST
         post("monophonic DSP");
     #endif
-    #if (DOWN_SAMPLING > 0)
-        #if (FILTER_TYPE == 0)
-            x->m_dsp = new dsp_down_sampler<Identity<Double<1,1>, DOWN_SAMPLING>>(new mydsp());
-        #elif (FILTER_TYPE == 1)
-            x->m_dsp = new dsp_down_sampler<LowPass3<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-        #elif (FILTER_TYPE == 2)
-            x->m_dsp = new dsp_down_sampler<LowPass4<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-        #elif (FILTER_TYPE == 3)
-            x->m_dsp = new dsp_down_sampler<LowPass3e<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-        #elif (FILTER_TYPE == 4)
-            x->m_dsp = new dsp_down_sampler<LowPass6eé<Double<45,100>, DOWN_SAMPLING, double>>(new mydsp());
-        #else
-            #error "ERROR : Filter type must be in [0..4] range"
-        #endif
-    #elif (UP_SAMPLING > 0)
-        #if (FILTER_TYPE == 0)
-            x->m_dsp = new dsp_up_sampler<Identity<Double<1,1>, UP_SAMPLING>>(new mydsp());
-        #elif (FILTER_TYPE == 1)
-            x->m_dsp = new dsp_up_sampler<LowPass3<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-        #elif (FILTER_TYPE == 2)
-            x->m_dsp = new dsp_up_sampler<LowPass4<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-        #elif (FILTER_TYPE == 3)
-            x->m_dsp = new dsp_up_sampler<LowPass3e<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-        #elif (FILTER_TYPE == 4)
-            x->m_dsp = new dsp_up_sampler<LowPass6e<Double<45,100>, UP_SAMPLING, double>>(new mydsp());
-        #else
-            #error "ERROR : Filter type must be in [0..4] range"
-        #endif
-    #else
-        x->m_dsp = new mydsp();
-    #endif
+        // Create a DS/US + Filter adapted DSP
+        x->m_dsp = createSRAdapter<FAUSTFLOAT>(new mydsp(), DOWN_SAMPLING, UP_SAMPLING, FILTER_TYPE);
     }
     
 #ifdef MIDICTRL
@@ -251,6 +227,32 @@ void faust_allocate(t_faust* x, int nvoices)
     if (sizeof(FAUSTFLOAT) == 4) {
         x->m_dsp = new dsp_sample_adapter<FAUSTFLOAT, double>(x->m_dsp);
     }
+    
+    x->m_Inputs = x->m_dsp->getNumInputs();
+    x->m_Outputs = x->m_dsp->getNumOutputs();
+    
+    // Initialize at the system's sampling rate
+    x->m_dsp->init(long(sys_getsr()));
+    
+    // Initialize User Interface (here connnection with controls)
+    x->m_dsp->buildUserInterface(x->m_dspUI);
+    
+#ifdef SOUNDFILE
+    x->m_dsp->buildUserInterface(x->m_soundInterface);
+#endif
+    
+    // Prepare JSON
+    faust_make_json(x);
+    
+    // Send JSON to JS script
+    faust_create_jsui(x);
+    
+#ifdef OSCCTRL
+    x->m_oscInterface = NULL;
+#endif
+    
+    // Load old controller state
+    x->m_dsp->buildUserInterface(x->m_savedUI);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -338,29 +340,7 @@ void faust_anything(t_faust* obj, t_symbol* s, short ac, t_atom* av)
 void faust_polyphony(t_faust* x, t_symbol* s, short ac, t_atom* av)
 {
     if (systhread_mutex_lock(x->m_mutex) == MAX_ERR_NONE) {
-        
-    #ifdef MIDICTRL
-        if (x->m_dsp_poly) {
-            x->m_midiHandler->removeMidiIn(x->m_dsp_poly);
-        }
-    #endif
-        
         faust_allocate(x, av[0].a_w.w_long);
-        
-        // Initialize at the system's sampling rate
-        x->m_dsp->init(long(sys_getsr()));
-        // Initialize User Interface (here connnection with controls)
-        x->m_dsp->buildUserInterface(x->m_dspUI);
-        
-        // Prepare JSON
-        faust_make_json(x);
-        
-        // Send JSON to JS script
-        faust_create_jsui(x);
-        
-        // Load old controller state
-        x->m_dsp->buildUserInterface(x->m_savedUI);
-        
         systhread_mutex_unlock(x->m_mutex);
     } else {
         post("Mutex lock cannot be taken...");
@@ -417,13 +397,13 @@ void faust_create_jsui(t_faust* x)
 /*--------------------------------------------------------------------------*/
 void faust_update_outputs(t_faust* x)
 {
-    for (auto& it1 : x->m_output_table) {
+    for (const auto& it1 : x->m_output_table) {
         bool new_val = false;
         FAUSTFLOAT value = x->m_dspUI->getOutputValue(it1.first, new_val);
         if (new_val) {
             t_atom at_value;
             atom_setfloat(&at_value, value);
-            for (auto& it2 : it1.second) {
+            for (const auto& it2 : it1.second) {
                 object_method_typed(it2, gensym("float"), 1, &at_value, 0);
             }
         }
@@ -446,47 +426,41 @@ void* faust_new(t_symbol* s, short ac, t_atom* av)
 {
     bool midi_sync = false;
     int nvoices = 0;
+    t_faust* x = (t_faust*)object_alloc(faust_class);
     
     mydsp* tmp_dsp = new mydsp();
     MidiMeta::analyse(tmp_dsp, midi_sync, nvoices);
+ #ifdef SOUNDFILE
+    Max_Meta3 meta3;
+    tmp_dsp->metadata(&meta3);
+    string bundle_path_str = SoundUI::getBinaryPathFrom(meta3.fName);
+    if (bundle_path_str == "") {
+        post("Bundle_path '%s' cannot be found!", meta3.fName.c_str());
+    }
+    x->m_soundInterface = new SoundUI(bundle_path_str, -1, nullptr, sizeof(FAUSTFLOAT) == 8);
+ #endif
     delete tmp_dsp;
     
-    t_faust* x = (t_faust*)object_alloc(faust_class);
-    
     x->m_savedUI = new SaveLabelUI();
+    // allocation is done in faust_allocate
     x->m_dspUI = NULL;
     x->m_dsp = NULL;
-    x->m_dsp_poly = NULL;
     x->m_json = NULL;
     x->m_mute = false;
+    x->m_control_outlet = outlet_new((t_pxobject*)x, (char*)"list");
     
 #ifdef MIDICTRL
-    x->m_midiHandler = new midi_handler();
+    x->m_midi_outlet = outlet_new((t_pxobject*)x, NULL);
+    x->m_midiHandler = new max_midi(x->m_midi_outlet);
     x->m_midiUI = new MidiUI(x->m_midiHandler);
 #endif
     
-    x->m_dspUI = new mspUI();
-    
     faust_allocate(x, nvoices);
-    
-    x->m_Inputs = x->m_dsp->getNumInputs();
-    x->m_Outputs = x->m_dsp->getNumOutputs();
-    
-    x->m_control_outlet = outlet_new((t_pxobject*)x, (char*)"list");
-
-    // Initialize at the system's sampling rate
-    x->m_dsp->init(long(sys_getsr()));
-    // Initialize User Interface (here connnection with controls)
-    x->m_dsp->buildUserInterface(x->m_dspUI);
-    
-    t_max_err err = systhread_mutex_new(&x->m_mutex, SYSTHREAD_MUTEX_NORMAL);
-    if (err != MAX_ERR_NONE) {
+   
+    if (systhread_mutex_new(&x->m_mutex, SYSTHREAD_MUTEX_NORMAL) != MAX_ERR_NONE) {
         post("Cannot allocate mutex...");
     }
     
-    // Prepare JSON
-    faust_make_json(x);
-    
     int num_input;
     if (x->m_dspUI->isMulti()) {
         num_input = x->m_dsp->getNumInputs() + 1;
@@ -495,6 +469,7 @@ void* faust_new(t_symbol* s, short ac, t_atom* av)
     }
     
     x->m_args = (void**)calloc((num_input + x->m_dsp->getNumOutputs()) + 2, sizeof(void*));
+    
     /* Multi in */
     dsp_setup((t_pxobject*)x, num_input);
 
@@ -504,38 +479,14 @@ void* faust_new(t_symbol* s, short ac, t_atom* av)
     }
 
     ((t_pxobject*)x)->z_misc = Z_NO_INPLACE; // To assure input and output buffers are actually different
-
-#ifdef SOUNDFILE
-    Max_Meta3 meta3;
-    x->m_dsp->metadata(&meta3);
-    string bundle_path_str = SoundUI::getBinaryPathFrom(meta3.fName);
-    if (bundle_path_str == "") {
-        post("Bundle_path '%s' cannot be found!", meta3.fName.c_str());
-    }
-    x->m_soundInterface = new SoundUI(bundle_path_str);
-    // SoundUI has to be dispatched on all internal voices
-    if (x->m_dsp_poly) x->m_dsp_poly->setGroup(false);
-    x->m_dsp->buildUserInterface(x->m_soundInterface);
-    if (x->m_dsp_poly) x->m_dsp_poly->setGroup(true);
-#endif
-    
-#ifdef OSCCTRL
-    x->m_oscInterface = NULL;
-#endif
-    
-    // Send JSON to JS script
-    faust_create_jsui(x);
-    
-    // Load old controller state
-    x->m_dsp->buildUserInterface(x->m_savedUI);
-    
+ 
     // Display controls
 #ifdef POST
     x->m_dspUI->displayControls();
-#endif   
+#endif
+    
     // Get attributes values
     attr_args_process(x, ac, av);
-    
     return x;
 }
 
@@ -621,8 +572,7 @@ void faust_init(t_faust* x, t_symbol* s, short ac, t_atom* av)
  }
 
 /*--------------------------------------------------------------------------*/
-// Dump controllers as list of: [path, cur, init, min, max]
-void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
+void faust_dump_inputs(t_faust* x)
 {
     // Input controllers
     for (mspUI::iterator it = x->m_dspUI->begin2(); it != x->m_dspUI->end2(); it++) {
@@ -633,6 +583,11 @@ void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
         atom_setfloat(&myList[3], (*it).second->getMaxValue());
         outlet_list(x->m_control_outlet, 0, 4, myList);
     }
+}
+
+/*--------------------------------------------------------------------------*/
+void faust_dump_outputs(t_faust* x)
+{
     // Output controllers
     for (mspUI::iterator it = x->m_dspUI->begin4(); it != x->m_dspUI->end4(); it++) {
         t_atom myList[4];
@@ -644,6 +599,14 @@ void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
     }
 }
 
+/*--------------------------------------------------------------------------*/
+// Dump controllers as list of [path, cur, min, max]
+void faust_dump(t_faust* x, t_symbol* s, short ac, t_atom* av)
+{
+    faust_dump_inputs(x);
+    faust_dump_outputs(x);
+}
+
 /*--------------------------------------------------------------------------*/
 void faust_dblclick(t_faust* x, long inlet)
 {
@@ -657,18 +620,20 @@ void faust_assist(t_faust* x, void* b, long msg, long a, char* dst)
     if (msg == ASSIST_INLET) {
         if (a == 0) {
             if (x->m_dsp->getNumInputs() == 0) {
-                sprintf(dst, "(message) : Unused Input");
+                snprintf(dst, 512, "(messages)");
             } else {
-                sprintf(dst, "(signal) : Audio Input %ld", (a+1));
+                snprintf(dst, 512, "(messages/signal) : Audio Input %ld", (a+1));
             }
         } else if (a < x->m_dsp->getNumInputs()) {
-            sprintf(dst, "(signal) : Audio Input %ld", (a+1));
+            snprintf(dst, 512, "(signal) : Audio Input %ld", (a+1));
         }
     } else if (msg == ASSIST_OUTLET) {
         if (a < x->m_dsp->getNumOutputs()) {
-            sprintf(dst, "(signal) : Audio Output %ld", (a+1));
+            snprintf(dst, 512, "(signal) : Audio Output %ld", (a+1));
+        } else if (a == x->m_dsp->getNumOutputs()) {
+            snprintf(dst, 512, "(list) : [path, cur|init, min, max]*");
         } else {
-            sprintf(dst, "(list) : [path, cur|init, min, max]*");
+            snprintf(dst, 512, "(int) : raw MIDI bytes*");
         }
     }
 }
@@ -719,7 +684,9 @@ void faust_perform64(t_faust* x, t_object* dsp64, double** ins, long numins, dou
         #ifdef OSCCTRL
             if (x->m_oscInterface) x->m_oscInterface->endBundle();
         #endif
-            faust_update_outputs(x);
+            //faust_update_outputs(x);
+            // Use the right outlet to output messages
+            faust_dump_outputs(x);
         }
     #if defined(MIDICTRL) || defined(OSCCTRL)
         GUI::updateAllGuis();
@@ -782,14 +749,13 @@ void ext_main(void* r)
     tmp_dsp->buildUserInterface(&tmp_UI);
     
     // Setup attributes
-    int i = 0;
     if (sizeof(FAUSTFLOAT) == 4) {
-        for (mspUI::iterator it = tmp_UI.begin1(); it != tmp_UI.end1(); it++, i++) {
+        for (mspUI::iterator it = tmp_UI.begin1(); it != tmp_UI.end1(); it++) {
             CLASS_ATTR_FLOAT(c, (*it).first.c_str(), 0, t_faust, m_ob);
             CLASS_ATTR_ACCESSORS(c, (*it).first.c_str(), NULL, (method)faust_attr_set);
         }
     } else {
-        for (mspUI::iterator it = tmp_UI.begin1(); it != tmp_UI.end1(); it++, i++) {
+        for (mspUI::iterator it = tmp_UI.begin1(); it != tmp_UI.end1(); it++) {
             CLASS_ATTR_DOUBLE(c, (*it).first.c_str(), 0, t_faust, m_ob);
             CLASS_ATTR_ACCESSORS(c, (*it).first.c_str(), NULL, (method)faust_attr_set);
         }
@@ -800,7 +766,7 @@ void ext_main(void* r)
     faust_class = c;
 #ifdef POST
     post((char*)"Faust DSP object v%s (sample = %s bits code = %s)", EXTERNAL_VERSION, ((sizeof(FAUSTFLOAT) == 4) ? "32" : "64"), getCodeSize());
-    post((char*)"Copyright (c) 2012-2020 Grame");
+    post((char*)"Copyright (c) 2012-2023 Grame");
 #endif
     Max_Meta1 meta1;
     tmp_dsp->metadata(&meta1);
@@ -818,5 +784,5 @@ void ext_main(void* r)
 #endif
 }
 
-/********************END ARCHITECTURE SECTION (part 2/2)****************/
+/******************* END max-msp64.cpp ****************/
 
